/*
 * Server code generation.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <stdio.h>
#include "types.h"



/* Attribute-configured signature and function definitions. */

void write_server_signature(struct signature *sig,
                            enum signature_role role,
                            FILE *fp, struct interface *iface);

void write_server_function(struct signature *sig, FILE *fp,
                           struct interface *iface);

/* Specific signature variant generation. */

void write_server_interface_signature(struct signature *sig,
                                      enum signature_role role, FILE *fp,
                                      struct interface *iface);

void write_server_wrapper_signature(struct signature *sig,
                                    enum signature_role role,
                                    FILE *fp, struct interface *iface);

void write_server_initiation_signature(struct signature *sig,
                                       enum signature_role role,
                                       FILE *fp, struct interface *iface);

void write_server_completion_signature(struct signature *sig,
                                       enum signature_role role,
                                       FILE *fp, struct interface *iface);

/* Wrapper function generation. */

void write_server_wrapper_function_body(struct parameter *param, FILE *fp,
                                        struct interface *iface,
                                        struct signature *sig);

void write_server_initiation_function_body(struct parameter *param, FILE *fp,
                                           struct interface *iface,
                                           struct signature *sig);

void write_server_completion_function_body(struct parameter *param, FILE *fp,
                                           struct interface *iface,
                                           struct signature *sig);

/* Common function body code generation. */

void write_superfluous_message_cast(FILE *fp, int input_words, int input_items,
                                    int output_words, int output_items);

void write_input_initialisation(struct parameter *param, FILE *fp,
                                const char *opname, int input_words,
                                int input_items);

void write_output_initialisation(struct parameter *param, FILE *fp,
                                 const char *opname, int output_words,
                                 int output_items, enum function_role function);

void write_server_function_call(struct parameter *param, FILE *fp,
                                struct interface *iface, struct signature *sig,
                                enum specifier specifier);
